<?php

	////////////////////////////////////////////////////////////////////////////////
	//
	// b2 0.6 to WordPress version 2.0.2 migration script
	// Written by Aral Balkan (http://aralbalkan.com, aral@aralbalkan.com)
	// Released under CC-GNU GPL (http://creativecommons.org/licenses/GPL/2.0/)
	//
	// Please note that this script is in no way guaranteed to work
	// for your particular installations of b2 and WordPress. 
	// Make sure you have backups of all your databases before
	// trying it out and test it out on your a non-deployment machine
	// first! 
	//
	// USE AT YOUR OWN RISK! :)
	//
	////////////////////////////////////////////////////////////////////////////////

	// Include ezSQL core
	include_once 'ez_sql_core.php';

	// Include ezSQL database specific component
	include_once 'ez_sql_mysql.php';

	// Include WP functions for sanitization
	include_once 'wp-includes/functions-formatting.php';
	
	////////////////////////////////////////////////////////////////////////////////
	//
	// Enter the connection info for your b2 and wordpress databases below
	//
	////////////////////////////////////////////////////////////////////////////////

	// General database info 
	// Note: On my system, ez_sql didn't like my using the default root account
	// with no password. I'd recommend creating a db user account with a password
	// and granting it permissions for both your b2 and wp databases before running this script.
	$dbUser = '';
	$dbPass = '';
	$dbHost = '';
	
	// b2 database name
	$dbB2Database = '';	

	// Wordpress database name
	$dbWpDatabase = '';

	if ( $dbUser == '' || $dbPass == '' || $dbHost == '' || dbB2Database == '' || dbWpDatabase == '' )
	{
		exit('Please configure the b2towp.php script with your database connection details before running it.');  
	}
	
	////////////////////////////////////////////////////////////////////////////////
	//
	// DO NOT CHANGE ANYTHING BELOW THIS LINE UNLESS YOU KNOW WHAT YOU'RE DOING!
	//
	////////////////////////////////////////////////////////////////////////////////

	$values = ' VALUES (';
	$end_values = ')';

	// Format a string for the database (used in INSERTs)
	function s ( $strForDb )
	{
		return "'$strForDb'";  
	}

	// Tack a comma on to the end
	function c ( $str )
	{
		return $str.', ';  
	}

	// Format a string for the database (used in INSERTs) with trailing comma
	function sc ( $strForDb )
	{
		return c ( s ( $strForDb ) );  
	}

	// Convert smart quotes to simple quotes
	function smartToSimpleQuotes ( $str )
	{
		$str = str_replace('', '\"', $str); 
		$str = str_replace('', '\"', $str); 
		$str = str_replace('', "\'", $str); 
		$str = str_replace('', "\'", $str); 
		
		return $str;
	}
	
	// Insert meta data into the user meta data table
	function insertWpUserMeta ( $user_id, $key, $value )
	{
	  	global $values;
	  	global $end_values;
	  	global $db;
	  	global $numInsertedUserMetaData;
	  	global $numErrors;
	  
  		$wpInsertUserMeta = 'INSERT INTO `wp_usermeta` (`user_id`, `meta_key`, `meta_value`)'
		.$values
			.c($user_id)
			.sc($key)
			.s($value)
		.$end_values;
	  
	  	$numRows = $db->query ( $wpInsertUserMeta );
	  	
		if ($numRows)
		{
			$numInsertedUserMetaData++;
		} 
		else
		{
		  	$numErrors++;
		  	$db->debug();
		} 		  
	}

	////////////////////////////////////////////////////////////////////////////////
	//
	// Batch script
	//
	////////////////////////////////////////////////////////////////////////////////

	echo '<h1>b2 to Wordpress migration script</h1>';
	echo '<i>Copyright (c) 2006 Aral Balkan. <a href="http://aralbalkan.com">http://aralbalkan.com</a>. Released under the open source MIT License</i><br /><br /><br />';
	//
	// Make initial database connection
	//
	$db = new ezSQL_mysql( $dbUser , $dbPass , $dbB2Database , $dbHost );

	//
	// A. b2 Database - Get the data
	//

	echo '<b>1. Reading data from the b2 database:</b> <br /><br />';


	// 1. Get the posts
	echo 'Reading posts from b2 database...';
	$b2SelectPosts = 'SELECT `ID`, `post_author`, `post_date`, `post_content`, `post_title`, `post_category` FROM `b2posts` WHERE 1';
	$b2Posts  = $db->get_results ( $b2SelectPosts );
	echo ' read ' . $db->num_rows . ' posts. <br />';
	
	
	// 2. Get the comments
	echo 'Reading comments from b2 database...';
	$b2SelectComments = 'SELECT `comment_ID`, `comment_post_ID`, `comment_author_ID`, `comment_author`, `comment_author_email`, `comment_author_url`, `comment_author_IP`, `comment_date`, `comment_content` FROM `b2comments` WHERE 1';
	$b2Comments = $db->get_results ( $b2SelectComments);
	echo ' read ' . $db->num_rows . ' comments. <br />';
	
	// 3. Get the categories
	echo 'Reading categories from b2 database...';
	$b2SelectCategories = 'SELECT `cat_ID`, `cat_name` FROM `b2categories` WHERE 1';
	$b2Categories = $db->get_results ( $b2SelectCategories );
	echo ' read ' . $db->num_rows . ' categories. <br />';
	
	// 4. Get the users
	echo 'Reading users from b2 database...';	
	$b2SelectUsers = 'SELECT `ID`, `user_login`, `user_pass`, `user_firstname`, `user_lastname`, `user_nickname`, `user_email`, `user_url`, `dateYMDhour`, `user_level`, `user_aim`, `user_yim` FROM `b2users` WHERE 1';		
	$b2Users = $db->get_results ( $b2SelectUsers );
	echo ' read ' . $db->num_rows . ' users. <br />';
	
	
	//
	// B. Wordpress database, convert and insert the data	
	//

	echo '<br /><b>2. Updating the Wordpress database:</b> <br /><br />';
		
	// Connect to Wordpress database
	$wp = $db->select ( $dbWpDatabase );
	
	//
	// 1. Categories
	//
	
	// a. Delete existing ones (if any)
	echo 'Clearing out the categories table... ';
	
	$wpDeleteCategories = 'DELETE FROM `wp_categories`';
	$numRows = $db->query($wpDeleteCategories);
	
	echo $numRows ? 'done!' : 'failed.';
	echo '<br />';

	// b. Insert categories	
	echo 'Inserting categories';
	$numCatsInserted = 0;
	$numErrors = 0;
	
	foreach ( $b2Categories as $b2Category )
	{
	  	$cat_id = $b2Category->cat_ID;
	  	$cat_name = $b2Category->cat_name;
	  	$category_nicename = sanitize_title_with_dashes ( $cat_name );
	  	
	  	$wpInsertCategories = 'INSERT INTO `wp_categories` (`cat_ID`, `cat_name`, `category_nicename`)'
			.$values
				.c($b2Category->cat_ID)
				.sc($cat_name)
				.s($category_nicename)
			.$end_values;
	
		// Debug			
		// echo	$wpInsertCategories . '<br />';
		
		$numRows = $db->query ( $wpInsertCategories );
		
		if ($numRows)
		{
			$numCatsInserted++;
		} 
		else
		{
		  	$numErrors++;
		  	$db->debug();
		} 		  
	}	
	
	echo ' '.$numCatsInserted . ' categories inserted. ('.$numErrors.' errors.)<br />';
	
	//
	// 2. Insert the posts and update the post2cat and categories count
	//
	
	// a. Clear the posts and post2cat tables
	echo 'Clearing the posts table... ';

	$wpDeletePosts = 'DELETE FROM `wp_posts`';	
	$numRows = $db->query($wpDeletePosts);
	
	echo $numRows ? 'done!' : 'failed.';
	echo '<br />';

	echo 'Clearing the post2cat relations table... ';
	
	$wpDeletePost2Cat = 'DELETE FROM `wp_post2cat`';
	$numRows = $db->query($wpDeletePost2Cat);
	
	echo $numRows ? 'done!' : 'failed.';
	echo '<br />';
	
	// b. Insert posts and handle category relations
	echo 'Inserting posts, updating category counts and post2cat relations';

	// Stats
	$numInsertedPosts = 0;
	$numUpdatedCategoryCounts = 0;
	$numInsertedPost2CatRelations = 0;
	$numErrors = 0;
	
	foreach ( $b2Posts as $b2Post )
	{
	  
	  	$post_content = $b2Post->post_content;	  	
	  	$post_title = $b2Post->post_title;
		$post_ID = $b2Post->ID;
	  	$post_category = $b2Post->post_category;
		$post_date = $b2Post->post_date;

		//
		// Do some cleanup of existing data 
		//

		// Remove curly quotes from existing content (Wordpress automatically 
		// renders these from regular quotes and chokes otherwise.)  
		$post_content = smartToSimpleQuotes ( $post_content );
		$post_title = smartToSimpleQuotes ( $post_title );

		// In case there are non-escaped quotes, do a search/replace
		$post_content = preg_replace('/[^\\\\]\'/', "\'", $post_content);
		$post_title = preg_replace('/[^\\\\]\'/', "\'", $post_title);
				

		////////////////////////////////////////
		// WP Insert Post SQL
		////////////////////////////////////////	  
		$wpInsertPost = 'INSERT INTO `wp_posts` (`ID`, `post_author`, `post_date`, `post_date_gmt`, `post_content`, `post_title`, `post_category`, `post_status`, `comment_status`, `post_name`, `post_modified`, `post_modified_gmt`)'
		  	.$values
			  	.c($post_ID)
			  	.c($b2Post->post_author)
			  	.sc($post_date)
			  	.sc($post_date)
			  	.sc($post_content)
			  	.sc($post_title)
			  	.c($post_category)
			  	.sc('publish')
			  	.sc('open')
			  	.sc(sanitize_title_with_dashes($post_title))
			  	.sc($post_date)
			  	.s($post_date)
		  	.$end_values;

		// Debug
		// echo $wpInsertPost . '<br />';

		$numRows = $db->query ( $wpInsertPost );
		
		if ($numRows)
		{
			$numInsertedPosts++;
		} 
		else
		{
		  	$numErrors++;
		  	$db->debug();
		} 		  
	
		////////////////////////////////////////
		// WP Update Category Count SQL
		////////////////////////////////////////
		$wpUpdateCategoryCount = 'UPDATE `wp_categories` SET category_count=category_count+1 WHERE cat_ID='.$post_category;
		
		// Debug
		// echo $wpUpdateCategoryCount . '<br />';

		$numRows = $db->query ( $wpUpdateCategoryCount );
		
		if ($numRows)
		{
			$numUpdatedCategoryCounts++;
		} 
		else
		{
		  	$numErrors++;
		} 		  

		
		////////////////////////////////////////
		// WP Insert Post2Cat SQL
		////////////////////////////////////////
		$wpInsertPost2Cat = 'INSERT INTO `wp_post2cat` (`post_id`, `category_id`)'
			.$values
				.c($post_ID)
				.$post_category
			.$end_values;
		
		// Debug
		// echo $wpInsertPost2Cat . '<br /><br />';

		$numRows = $db->query ( $wpInsertPost2Cat );
		
		if ($numRows)
		{
			$numInsertedPost2CatRelations++;
		} 
		else 
		{
		  	$numErrors++;
		  	$db->debug();
		} 		  
		
	}
	
	// Print final stats for Posts-related updates
	echo '<br />';
	echo $numInsertedPosts . ' posts inserted. <br />';
	echo $numUpdatedCategoryCounts . ' category counts updated. <br />';
	echo $numInsertedPost2CatRelations . ' post2cat relations inserted. <br />';
	echo '('. $numErrors . ' errors.) <br /><br />';
	  
	//	
	// 3. Insert the comments
	//

	// a. Clear the comments table
	echo 'Clearing the comments table... ';

	$wpDeleteComments = 'DELETE FROM `wp_comments`';	
	$numRows = $db->query($wpDeleteComments);
	
	echo $numRows ? 'done!' : 'failed.';
	echo '<br />';

	// Stats
	$numInsertedComments = 0;
	$numUpdatedCommentCounts = 0;
	$numErrors = 0;

	// b. Update the comments table
	echo 'Inserting comments and updating comment counts in posts table';

	foreach ( $b2Comments as $b2Comment )
	{
	  	// i. Insert the comments
		
		$comment_ID = $b2Comment->comment_ID;
		$comment_content = $b2Comment->comment_content;
		$comment_author = $b2Comment->comment_author;
		$comment_post_ID = $b2Comment->comment_post_ID;

		// On some entries, the first character of the author name field
		// is appearing as ASCII 160 (HTML space) and this is being 
		// displayed as a question mark in the comments at the head of the 
		// name. Here we check for these cases and strip out the character.
		$comment_author = ( ord($comment_author[0]) == 160 ) ? substr ( $comment_author, 1, strlen($comment_author)-1 ) : $comment_author;

		// If there are html entities in the names (eg. accented characters),
		// make sure that these are correctly handled.
		$comment_author = htmlentities( $comment_author );

		// echo $comment_author . ' - ' ;
		//echo ord($comment_author[0]).'<br /><br />';
		
		//
		// Do some cleanup of existing data 
		//

		// Remove curly quotes from existing content (Wordpress automatically 
		// renders these from regular quotes and chokes otherwise.)  
		$comment_content = smartToSimpleQuotes ( $b2Comment->comment_content );

		// In case there are non-escaped quotes, do a search/replace
		$comment_content = preg_replace('/[^\\\\]\'/', "\'", $comment_content);
		$comment_content = preg_replace('/[^\\\\]\"/', '\"', $comment_content);

		$comment_author = preg_replace('/[^\\\\]\'/', "\'", $comment_author);
		$comment_author = preg_replace('/[^\\\\]\"/', '\"', $comment_author);

		////////////////////////////////////////
		// WP Insert Comment SQL
		////////////////////////////////////////		
		$wpInsertComments = 'INSERT INTO `wp_comments` (`comment_ID`, `comment_post_ID`, `comment_author`, `comment_author_email`, `comment_author_url`, `comment_author_IP`, `comment_date`, `comment_date_gmt`, `comment_content`, `user_id`)'
			.$values
				.c($comment_ID)
				.c($comment_post_ID)
				.sc($comment_author)
				.sc($b2Comment->comment_author_email)
				.sc($b2Comment->comment_author_url)
				.sc($b2Comment->comment_author_IP)
				.sc($b2Comment->comment_date)
				.sc($b2Comment->comment_date)
				.sc($comment_content)
				.$b2Comment->comment_author_ID		
			.$end_values;

			// Debug
			//echo $wpInsertComments . '<br /><br />';
		
			$numRows = $db->query ( $wpInsertComments );
			
			if ($numRows)
			{
				$numInsertedComments++;
			} 
			else 
			{
			  	$numErrors++;
			  	$db->debug();
			} 		  
				
		// ii. Update the comment count on the posts table

		////////////////////////////////////////
		// WP Update Comment Count SQL
		////////////////////////////////////////
		$wpUpdateCommentCount = 'UPDATE `wp_posts` SET comment_count=comment_count+1 WHERE ID='.$comment_post_ID;
		
		// Debug
		// echo $wpUpdateCommentCount . '<br />';

		$numRows = $db->query ( $wpUpdateCommentCount );
		
		if ($numRows)
		{
			$numUpdatedCommentCounts++;
		} 
		else
		{
		  	$numErrors++;
		  	$db->debug();
		} 		  
		
	}	

	// Print final stats for comments-related updates
	echo '<br />';
	echo $numInsertedComments . ' comments inserted. <br />';
	echo $numUpdatedCommentCounts . ' comment counts updated. <br />';
	echo '('. $numErrors . ' errors.) <br /><br />';
	
	//
	// 4. Insert the users and user meta data
	//

	// Stats
	$numInsertedUsers = 0;
	$numInsertedUserMetaData = 0;
	$numErrors = 0;

	// a. Clear the existing tables
	echo 'Clearing the users table (except for user 1, not touching admin)... ';

	$wpDeleteUsers = 'DELETE FROM `wp_users` WHERE ID>1';	
	$numRows = $db->query($wpDeleteUsers);
	
	echo $numRows ? 'done!' : 'failed.';
	echo '<br />';
		
	echo 'Clearing the user meta data table (except for user 1, not touching admin)... ';

	$wpDeleteUserMetaData = 'DELETE FROM `wp_usermeta` WHERE user_id>1';	
	$numRows = $db->query($wpDeleteUserMetaData);
	
	echo $numRows ? 'done!' : 'failed.';
	echo '<br />';

	// b. Insert the users and user meta data
	echo 'Inserting users and user meta data';

	foreach ( $b2Users as $b2User )
	{
		$user_id = $b2User->ID;

	  	// We are not touching the first account (admin)
	  	if ( $user_id > 1 )
	  	{
	  	  	$user_login = $b2User->user_login;
	  	  
	  	  	$user_firstname = htmlentities($b2User->user_firstname);
	  	  	$user_lastname = htmlentities($b2User->user_lastname);
	  	  	
	  	  	// Try to set the display name as the user's full name but
	  	  	// if user didn't provide the data, use her login instead.
	  	  	$display_name = $user_firstname.' '.$user_lastname;
	  	  	if ( $display_name == ' ') $display_name = $user_login;
	  	  
			////////////////////////////////////////
			// WP Insert User SQL
			////////////////////////////////////////
			$wpInsertUser = 'INSERT INTO `wp_users` (`ID`, `user_login`, `user_pass`, `user_nicename`, `user_email`, `user_url`, `user_registered`, `display_name`)'
				.$values
					.c($user_id)
					.sc($user_login)
					.sc(md5($b2User->user_pass))
					.sc(sanitize_title_with_dashes($user_login))
					.sc($b2User->user_email)
					.sc($b2User->user_url)
					.sc($b2User->dateYMDhour)
					.s($display_name) 
				.$end_values;

			// Debug
			// echo $wpInsertUser . '<br />';
			
			$numRows = $db->query ( $wpInsertUser );
			
			if ($numRows)
			{
				$numInsertedUsers++;
			} 
			else 
			{
			  	$numErrors++;
			  	$db->debug();
			} 		  

			////////////////////////////////////////
			// WP Insert User Meta Data SQL
			////////////////////////////////////////
			insertWpUserMeta ( $user_id, 'first_name', $user_firstname );
			insertWpUserMeta ( $user_id, 'last_name', $user_lastname );
			insertWpUserMeta ( $user_id, 'nickname', $b2User->user_nickname );
			insertWpUserMeta ( $user_id, 'aim', $b2User->user_aim );
			insertWpUserMeta ( $user_id, 'yim', $b2User->user_yim );
			
			// Set all user levels to zero and make them all subscribers.
			insertWpUserMeta ( $user_id, 'wp_user_level', '0');	 
			insertWpUserMeta ( $user_id, 'wp_capabilities', 'a:1:{s:10:"subscriber";b:1;}');

			// Although these are empty, we need to have a full set of metadata or 
			// the login doesn't work corrently
			insertWpUserMeta ( $user_id, 'jabber', '' );
			insertWpUserMeta ( $user_id, 'description', '' );

		}		
	}
	
	// Print final stats for User-related updates
	echo '<br />';
	echo $numInsertedUsers . ' users inserted. <br />';
	echo $numInsertedUserMetaData . ' items of user meta data inserted. <br />';
	echo '('. $numErrors . ' errors.) <br /><br />';
		
?>